<?php

/*
 * General-purpose helpers for WP, which could theoretically migrate to a bt-helpers plugin for all sites.
 */


/**
 * Helper to get singular or plural display name of a post type
 *
 * @param \WP_post|null $post
 * @param bool $plural Set to true of you want plural. Default false/singular.
 *
 * @return string
 */
if ( ! function_exists( 'bt_get_post_type_name' ) ) {
	function bt_get_post_type_name( $post = null, $plural = false ) {
		if ( $post_type = get_post_type_object( get_post_field( 'post_type', $post ) ) ) {
			//error_log( __METHOD__ . ': post = ' . $post->post_title . ' // type = ' . get_post_field( 'post_type', $post ) );
			//error_log( __METHOD__ . ': post_type = ' . print_r( $post_type, 1 ) );
			if ( $plural ) {
				return $post_type->labels->name;
			} else {
				return $post_type->labels->singular_name;
			}
		}
	}
}


/**
 * Helper to determine what a post's author id is
 *
 * @param $post_id
 *
 * @return string
 */
if ( ! function_exists( 'bt_get_author_id_for_post' ) ) {
	function bt_get_author_id_for_post( $post_id = null ) {
		return get_post_field( 'post_author', $post_id );
	}
}


/**
 * Get the count of a user's posts of a given type, and optionally of a given status list, and optionally
 * since/until dates.
 *
 * Only works for product and shop_order post types for now.
 *
 * @param int $user_id
 * @param string $post_type
 * @param string $post_status A list which can go in an "IN (___)" sql statement
 * @param string $since
 * @param string $until
 * @param string $meta_join SQL clause to include a join for meta data
 * @param string $meta_where SQL clause to include a condition for meta data
 *
 * @return string|null
 * @todo Prepare query vars properly.
 *
 */
function bt_get_user_post_count( $user_id, $post_type, $post_status = '', $since = '', $until = '', $meta_join = '', $meta_where = '' ) {
	if ( ! is_numeric( $user_id ) ) {
		wp_die( __METHOD__ . ': no user ID provided.' );
	}

	switch ( $post_type ) {
		case 'product':
			if ( ! $post_status ) {
				$post_status = "'publish','draft'";
			}
			break;
		case 'shop_order':
			if ( ! $post_status ) {
				$post_status = "'wc-completed','wc-processing','wc-refunded','wc-failed','wc-ready-pickup','wc-completed-picked','wc-completed-shipped'";
			}
			break;
		default:
			wp_die( __METHOD__ . ': invalid post type: ' . $post_type );
			return '';
	}

	global $wpdb;

	$query = "
            SELECT COUNT( ID )
            AS num_posts
            FROM {$wpdb->posts}
            	$meta_join
            WHERE 1=1
            	AND {$wpdb->posts}.post_author = $user_id
                AND {$wpdb->posts}.post_type = '$post_type'
                AND ({$wpdb->posts}.post_status IN ($post_status))
            ";
	if ( $meta_join && $meta_where ) {
		$query .= " AND( $meta_where )";
	}

	if ( ! empty( $since ) ) {
		$query .= " AND {$wpdb->posts}.post_date >= '$since'";
		//error_log( 'SELLER POST COUNT: query = ' . $query );
	}

	if ( ! empty( $until ) ) {
		$query .= " AND {$wpdb->posts}.post_date < '$until'";
		//error_log( 'SELLER POST COUNT: query = ' . $query );
	}

	//error_log( 'SELLER POST COUNT: query = ' . $query );

	return $wpdb->get_var( $query );

}


/**
 * A function like wp_sideload_image() but allows PDFs too.
 *
 * @param $file
 * @param int $post_id
 * @param null $desc
 *
 * @return int|WP_Error
 */
if ( ! function_exists( 'bt_sideload_file' ) ) {
	function bt_sideload_file( $file, $post_id = 0, $desc = null ) {
		if ( empty( $file ) ) {
			return new \WP_Error( 'error', 'File is empty' );
		}

		$file_array = array();

		// Get filename and store it into $file_array
		// Add more file types if necessary
		preg_match( '/[^\?]+\.(jpe?g|jpe|gif|png|pdf)\b/i', $file, $matches );
		$file_array['name'] = basename( $matches[0] );

		// Download file into temp location.
		$file_array['tmp_name'] = download_url( $file );

		// If error storing temporarily, return the error.
		if ( is_wp_error( $file_array['tmp_name'] ) ) {
			return new \WP_Error( 'error', 'Error while storing file temporarily' );
		}

		// Store and validate
		$id = media_handle_sideload( $file_array, $post_id, $desc );

		// Unlink if couldn't store permanently
		if ( is_wp_error( $id ) ) {
			unlink( $file_array['tmp_name'] );

			return new \WP_Error( 'error', "Couldn't store upload permanently" );
		}

		if ( empty( $id ) ) {
			return new \WP_Error( 'error', "Upload ID is empty" );
		}

		return $id;
	}
}


if ( ! function_exists( 'bt_is_us_phone_valid' ) ) {
	/**
	 * Test if a provided phone number matches US requirements.
	 *
	 * @param string $phone
	 *
	 * @return false|int
	 */
	function bt_is_us_phone_valid( $phone ) {
		return preg_match(
			'/(?:(?:(\s*\(?([2-9]1[02-9]|[2-9][02-8]1|[2-9][02-8][02-9])\s*)|([2-9]1[02-9]|[2-9][02-8]1|[2-9][02-8][02-9]))\)?\s*(?:[.-]\s*)?)([2-9]1[02-9]|[2-9][02-9]1|[2-9][02-9]{2})\s*(?:[.-]\s*)?([0-9]{4})/',
			$phone
		);
	}
}


/**
 * Do some cleanup/normalization of phone numbers, trying to get them to be like 612-666-0000
 *
 * @param $phone
 *
 * @param bool $add_hyphens default true
 *
 * @return mixed|string
 */
function clean_phone( $phone, $add_hyphens = true ) {
  // strip out chars that need to be normalized into hyphens (this borks extensions or additional info...)
  $phone = str_replace( ['(', ')', '.', ' ', '-'], '', $phone );
  // strip a leading 1
  if ( substr( $phone, 0, 1 ) == '1' ) {
    $phone = ltrim( $phone, '1' );
  }
  // get hyphens in where we want them if we have a 10-digit number after cleanup
  if ( $add_hyphens && strlen( $phone ) == 10 ) {
    $phone = substr( $phone, 0, 3 ) . '-' . substr( $phone, 3, 3 ) . '-' . substr( $phone, 6, 4 );
  }
  return $phone;
}


/**
 * If need be, wrap text for a CSV value in double quotes and escape any contained double quotes.
 *
 * @param $text
 *
 * @return string
 */
function bt_maybe_quote( $text ) {
	// trim first
	$text = trim( $text );
	// look for commas, double quotes, or EOLs
	if ( strpos( $text, ',' ) !== false || strpos( $text, '"' ) !== false || strpos( $text, PHP_EOL ) !== false ) {
		//error_log('ORIGINAL TEXT: ' . $text);
		$text = '"' . str_replace( '"', '\\"', $text ) . '"';
		//error_log('UPDATED TEXT: ' . $text);
	}
	return $text;
}


/**
 * Generate a CSV from data, which has column headers as keys for each item.
 *
 * @param $data
 *
 * @return string
 */
function bt_csv_from_data( $data ) {
	$csv = '';
	$cols = array_keys( current( $data ) );
	// - header row
	foreach ( $cols as $col ) {
		$csv .= bt_maybe_quote( $col ) . ',';
	}
	$csv = rtrim( $csv, ',' ) . PHP_EOL;
	// - all data rows
	foreach ( $data as $row ) {
		foreach ( $cols as $col ) {
			$csv .= bt_maybe_quote( $row[$col] ) . ',';
		}
		$csv = rtrim( $csv, ',' ) . PHP_EOL;
	}

	return $csv;
}
