<?php

/**
 * Helper to see if an order's shipping address matches its billing address. Used e.g. to alert for potential fraud
 *
 * @param \WC_Order $order
 *
 * @return bool true if no shipping or if address parts match
 */
function bt_order_shipping_matches_billing( $order ) {
	//error_log( __METHOD__ . ': ship addr 1 = ' . $order->get_shipping_address_1() );

	if ( empty( $order->get_shipping_address_1() ) ) {
		// no shipping involved, so we consider this true
		return true;
	}

	$billing_address = strtolower( $order->get_billing_address_1() );
	$billing_city = strtolower( $order->get_billing_city() );
	$billing_postcode = strtolower( $order->get_billing_postcode() );
	$billing_state = strtolower( $order->get_billing_state() );
	$shipping_address = strtolower( $order->get_shipping_address_1() );
	$shipping_city = strtolower( $order->get_shipping_city() );
	$shipping_postcode = strtolower( $order->get_shipping_postcode() );
	$shipping_state = strtolower( $order->get_shipping_state() );

	if (
		$billing_address != $shipping_address
		|| $billing_city != $shipping_city
		|| $billing_postcode != $shipping_postcode
		|| $billing_state != $shipping_state
	) {
		return false;
	}

	// everything matched
	return true;
}


/**
 * Text for sellers to be alerted to fraud prevention measures.
 *
 * @return string
 * /
function bt_shipping_fraud_prevention_message() {
	return '
		<p><strong>Heads Up!</strong></p>
		<p>
			This order is to ship to someone other than the purchaser.
		</p>
		<p>
			If the order is of high value, please verify that the addresses are real and contact the customer 
			to confirm they have made the purchase.
		</p>
		<p>
			Unfortunately, high-value items are a target for people with stolen credit cards, and these steps
			help prevent fraud and problematic chargebacks.
		</p>
';
} */


/**
 * Helper to determine if we're doing package rate calculations based on weight/dimensions.
 *
 * We consider "does usps" as ships, in addition to curbside, and assume that USPS would be the provider. However,
 * we may do rate calc or we may do flat rates using shipping classes that match pre-paid priority mail. So this
 * function tests to see if the USPS shipping plugin is in use by seeing if its core class exists. If it does, we
 * are doing rate calc. If not, we are doing flat rate.
 *
 * @return bool
 */
function usps_use_rate_calc() {
	return class_exists( '\WC_USPS' );
}


/**
 * Helper to determine if a product can ship via USPS.
 *
 * If using rate calc, product requires dimensions and weight, and dimensions cannot exceed the
 * limit for USPS Retail Ground.
 *
 * If flat rate, it's just a matter of whether the seller does usps (ships in some form) or not.
 *
 * @param \WC_Product $product
 *
 * @return bool
 */
function bt_product_can_ship_usps( $product ) {
	// see if seller uses usps at all
	$seller_post = bt_get_seller_post( bt_get_author_id_for_post( $product->get_id() ) );
	if ( ! bt_seller_does_usps( $seller_post ) ) {
		//error_log( __METHOD__ . ': seller does NOT do USPS!' );
		return false;
	}

	if ( ! usps_use_rate_calc() ) {
		// flat rate rather than rate calc, and since the seller does usps this product can ship
		return true;
	}

	// make sure we have dimensions and weight for a rate calc
	if ( $product->has_dimensions() && $product->has_weight() ) {

		// check package size for max @see https://faq.usps.com/s/article/What-is-USPS-Retail-Ground-Service
		// this assumes Retail Ground service is allowed
		$limit = 130;
		$size = intval( $product->get_length() ) + ( 2 * $product->get_width() ) + ( 2 * $product->get_height() );

		//error_log( __METHOD__ . ': product ' . $product->get_title() . ' / size = ' . $size . ' / limit = ' . $limit . ' / dims = ' . $product->get_length() . 'x' . $product->get_width() . 'x' . $product->get_height() );

		if ( $size <= $limit ) {
			return true;
		}

	}

	return false;
}


/**
 * Helper to determine if the current cart is using curbside pickup for fulfillment. We assume this is true if we use
 * sellers, but we'll look for a chosen shipping method, which would exist in a site that allows shipping in addition
 * to curbside pickup.
 *
 * If there is a chosen shipping method and it isn't local_pickup (what we use for curbside pickup when shipping is
 * enabled), then this returns false.
 *
 * NOTE: in places where user can change shipping method, we can't use this helper unless we update our custom things
 * via ajax.
 *
 * @return bool
 */
function bt_cart_is_curbside_pickup() {
	$shipping_methods = wc_get_chosen_shipping_method_ids();
	//error_log( __METHOD__ . ' -- $shipping_methods = ' . print_r( $shipping_methods, true ) );
	if ( ! empty( $shipping_methods ) ) {
		if ( $shipping_methods[0] != 'local_pickup' ) {
			return false;
		}
	}

	return bt_use_sellers();
}


/**
 * Helper to determine if a cart can be shipped via USPS.
 *
 * If using rate calc, this loops products in cart and checks each with our helper
 * bt_product_can_ship_usps(). If we find an item that can't ship via USPS, we return false.
 *
 * If flat rate, it's just a matter of whether the seller does usps (ships in some form) or not.
 *
 * @return bool False if any item in cart can't ship via USPS, else true.
 */
function bt_cart_can_ship_usps() {
	if ( function_exists( 'WC') && ! empty( WC()->cart ) ) {
		// see if seller uses usps at all
		if ( ! bt_seller_does_usps() ) {
			//error_log( __METHOD__ . ': seller does NOT do USPS!' );
			return false;
		}

		if ( ! usps_use_rate_calc() ) {
			// flat rate rather than rate calc, and since the seller does usps the cart can ship
			//error_log( __METHOD__ . ': flat rate' );
			return true;
		}

		if ( ! empty( $products = WC()->cart->get_cart() ) ) {
			foreach ( $products as $product ) {
				//error_log( __METHOD__ . ': product = ' . print_r( $product, 1 ) );
				if ( ! bt_product_can_ship_usps( $product['data'] ) ) {
					// we found a product which cannot ship via usps
					//error_log( __METHOD__ . ': we found a product which cannot ship via usps' );
					return false;
				}
			}
		}
	}

	//error_log( __METHOD__ . ': assuming true!' );
	return true;
}


/**
 * Filter USPS shipping availability to ensure that all items have dimensions and weight in order to allow it.
 *
 * This only runs if using the formal USPS plugin for actual rate calculations.
 */
add_filter( 'woocommerce_shipping_usps_is_available', function( $is_available, $package ) {
	//error_log( 'USPS IS AVAIL: bool =  ' . ( $is_available ? 'true' : 'false' ) . ' / package = ' . print_r( $package, 1 ) );

	// check products to ensure we have dimensions and weight, which we require in order to allow USPS shipping,
	// along with max size for retail ground.
	foreach ( $package['contents'] as $key => $contents ) {
		//error_log( 'USPS IS AVAIL: item =  ' . $contents['data']->get_title() . ' / has weight = ' . ( $contents['data']->has_weight() ? 'true' : 'false' ) . ' / has dimensions = ' . ( $contents['data']->has_dimensions() ? 'true' : 'false' ) . '' );
		if ( ! bt_product_can_ship_usps( $contents['data'] ) ) {
			return false;
		}
	}

	return $is_available;
}, 9988, 2 );


/**
 * Filter package rates so that we:
 * - remove local pickup (curbside) if seller doesn't do curbside
 * - remove flat rate if actual usps rate calc is in place and the order can ship via that method, or if seller doesn't
 *      do usps
 * - if free shipping method exists, and seller uses delivery fees, calculate and set the delivery fee
 *
 * NOTE: this only runs on actions like cart update, not on every cart view.
 *
 * NOTE: "free_shipping" and "flat_rate" don't behave the same way, as the latter asks for address in order to display
 * the rate. So we use "free_shipping" for projects that need to use a calculated delivery fee, but DON'T use it in
 * projects that have local_pickup and flat_rate since we aren't doing anything so far with free_shipping in those.
 */
add_filter( 'woocommerce_package_rates', function( $rates ) {
	//error_log( 'PACKAGE RATES: ' . print_r( $rates, 1 ) );

	// loop packages to determine keys for the methods we need to deal with, which can be different between sites
	$curbside_method_key = null;
	$flat_rate_key = null;
	$free_shipping_key = null;
	foreach ( $rates as $key => $rate ) {
		if ( strpos( $key, 'local_pickup' ) !== false && empty( $curbside_method_key ) ) {
			// NOTE: we only use the first local_pickup option, because cml/nma has multiple, to facilitate 10x10 pickup days
			//error_log( ' -- setting $curbside_method_key = ' . $key );
			$curbside_method_key = $key;
		} else if ( strpos( $key, 'flat_rate' ) !== false /*&& empty( $flat_rate_key )*/ ) {
			//error_log( ' -- setting $flat_rate_key = ' . $key );
			$flat_rate_key = $key;
		} else if ( strpos( $key, 'free_shipping' ) !== false /*&& empty( $free_shipping_key )*/ ) {
			//error_log( ' -- setting $free_shipping_key = ' . $key );
			$free_shipping_key = $key;
		//} else {
			//error_log( ' -- DOING NOTHING WITH KEY = ' . $key );
		}
	}

	// remove curbside option if seller doesn't use it
	if ( ! bt_seller_does_curbside() && array_key_exists( $curbside_method_key, $rates ) ) {
		unset( $rates[$curbside_method_key] );
	}

	// remove flat rate option if...
	if ( usps_use_rate_calc() && bt_cart_can_ship_usps() && array_key_exists( $flat_rate_key, $rates ) ) {
		// ...using actual rate calc and order can ship via usps
    //error_log( ' -- UNsetting $flat_rate_key -- CASE A' );
		unset( $rates[$flat_rate_key] );
	} else if ( bt_use_sellers() && ! bt_seller_does_usps() ) {
		// ...seller does curbside only
    //error_log( ' -- UNsetting $flat_rate_key -- CASE B' );
		unset( $rates[$flat_rate_key] );
	} else if ( $flat_rate_key && $free_shipping_key ) {
    // ...both flat/standard and free shipping exist, and we don't want it left on Flat/Standard when Free is available
    unset( $rates[$flat_rate_key] );
  }

	// change price of "free shipping" method based on seller setup/config
	$delivery_fees = bt_ecomm_option( 'delivery_fees' );
	if ( $delivery_fees && array_key_exists( $free_shipping_key, $rates ) ) {
		$subtotal = WC()->cart->get_cart_contents_total();
		//error_log( __METHOD__ . ' -- cart subtotal = ' . $subtotal . ' -- fees = ' . print_r( $delivery_fees, true ) );
		foreach ( $delivery_fees as $fee_range ) {
			//error_log( __METHOD__ . ' -- checking fee range = ' . print_r( $fee_range, true ) );
			if ( $fee_range['min'] <= $subtotal && $subtotal <= $fee_range['max'] ) {
				// set the shipping cost to this fee
				$rates[$free_shipping_key]->set_cost( $fee_range['fee'] );
				// prepend FREE to the shipping method if no cost
				if ( $fee_range['fee'] == 0 ) {
					$rates[$free_shipping_key]->set_label(
						'FREE ' . $rates[$free_shipping_key]->get_label()
					);
				}
				// don't need to keep looping
				break;
			}
		}
	}

	//error_log( 'PACKAGE RATES ADJUSTED: ' . print_r( $rates, 1 ) );
	return $rates;
}, 9988, 1 );


/**
 * Filter the label of a shipping method so that:
 * - If flat rate shipping and cost is zero, call it "Free Shipping"
 */
add_filter( 'woocommerce_cart_shipping_method_full_label', function( $label, $method ) {
	//error_log( 'SHIP METHOD LABEL: label = ' . $label . ' / method = ' . print_r( $method, 1 ) );

	if ( $method->get_id() == 'flat_rate:2' && $method->get_cost() == 0 ) {
		$label = 'Free Shipping';
	}

	return $label;
}, 9988, 2 );


/**
 * Filter USPS shipping options so that:
 * - we set origin zip code to the seller zip if it exists
 */
add_filter( 'woocommerce_shipping_usps_option', function( $option, $key, $method ) {
	if ( $key == 'origin' ) {
		//error_log( 'USPS ORIGIN: option = ' . $option . ' / key = ' . $key /*. ' / method = ' . print_r( $method, 1 )*/ );

		$seller_zip = bt_seller_zip();
		//error_log( 'USPS ORIGIN: seller zip = ' . $seller_zip );

		if ( is_numeric( $seller_zip ) ) {
			$option = $seller_zip;
		}
	}

	return $option;
}, 9988, 3 );
